﻿namespace Example.Application.Services
{
    using Example.Core.Interfaces;
    using Microsoft.Extensions.Configuration;
    using System;

    /// <summary>
    /// The key service.
    /// </summary>
    public class KeyService : IKeyService
    {
        #region Private Fields

        /// <summary>
        /// The encryption constant.
        /// </summary>
        private readonly string encryptionConstant;

        /// <summary>
        /// The private key.
        /// </summary>
        private readonly string privateKey;

        /// <summary>
        /// The public key.
        /// </summary>
        private readonly string publicKey;

        #endregion Private Fields

        #region Public Constructors

        public KeyService(IConfiguration configuration)
        {
            this.encryptionConstant = configuration["Enigma:EncryptionConstant"];
            this.privateKey = configuration["Enigma:PrivateKey"];
            this.publicKey = configuration["Enigma:PublicKey"];
        }

        #endregion Public Constructors

        #region Public Methods

        /// <inheritdoc cref="IKeyService"/>
        public string GenerateKey(string name, string id, DateTime endDate)
        {
            var kg = this.GenerateKeyGenParams();

            kg.ExpirationDay = endDate.Day;
            kg.ExpirationMonth = endDate.Month;
            kg.ExpirationYear = endDate.Year;
            kg.RegInfo = name;
            kg.RegInfoLen = kg.RegInfo.Length;
            kg.HardwareID = id;

            var result = Enigma_KeyGen_IDE64.KG_GenerateRegistrationKey(ref kg);

            return kg.Key;
        }

        /// <inheritdoc cref="IKeyService"/>
        public bool ValidId(string id)
        {
            throw new Exception();
        }

        #endregion Public Methods

        #region Private Methods

        /// <summary>
        /// Generate a new TKeyGenParams.
        /// </summary>
        /// <returns></returns>
        private Enigma_KeyGen_IDE64.TKeyGenParams GenerateKeyGenParams()
        {
            var random = new Random();
            var kg = new Enigma_KeyGen_IDE64.TKeyGenParams
            {
                // Key details.
                KeyMode = Enigma_KeyGen_IDE64.RM_512,
                KeyBase = Enigma_KeyGen_IDE64.RB_8,
                KeyWithHyphens = false,
                UseKeyExpiration = true,

                // Encryption.
                PublicKey = this.publicKey,
                PrivateKey = this.privateKey,
                Key = new string('\0', 2048),
                EncryptedSections = new bool[] { false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false },

                // Locking methods.
                UseHardwareLocking = true,
                UseRegisterAfter = false,
                UseRegisterBefore = false,
                UseExecutionsLimit = false,
                UseDaysLimit = false,
                UseRunTimeLimit = false,
                UseGlobalTimeLimit = false,
                UseCountyLimit = false
            };

            kg.KeyLen = kg.Key.Length;
            kg.EncryptedSections[random.Next(0, 15)] = true;

            return kg;
        }

        #endregion Private Methods
    }
}