﻿namespace Example.Application.Tests.Services
{
    using Example.Application.Services;
    using Microsoft.Extensions.Configuration;
    using System;
    using System.Collections.Generic;
    using Xunit;

    /// <summary>
    /// The key service tests.
    /// </summary>
    public class KeyServiceTests
    {
        #region Private Fields

        /// <summary>
        /// The enigma config.
        /// </summary>
        private static readonly Dictionary<string, string> EnigmaConfig = new Dictionary<string, string>
                                                                              {
                                                                                  {
                                                                                      "Enigma:PublicKey",
                                                                                      "020690B756D7B03CB420D07A85B26850A50067B4UPJ4CKAM6CERAV89T329RV885X"
                                                                                      + "KQFN43X42UDBEWG7EE8UGT56K34SX2M3SXTF8N4N3RFXP98HY65ZBDV5PZ9JDBDJ"
                                                                                      + "24HW6NSZNBT067BDQLD8LQDRUQHM3L7G2A9M2M7NPG3W7TCNKN9N655Z9RYZZATC"
                                                                                      + "PR3TFTZSTTJ4963H6GNUKLNRURP4D6A587BJHUGRFNKX7HVURDQSQ067CCHUWSGY"
                                                                                      + "YUQ3KUT72EL2LV68JY7FTCGZX6HTPLMG4ZFYLGPUJ7THYM2AC4E8DFZH8ECGRNF2"
                                                                                      + "A7629LJPPB75BPFZ35NUJQ2AXX84XWTM"
                                                                                  },
                                                                                  {
                                                                                      "Enigma:PrivateKey",
                                                                                      "00DDYQWVZ8BC5BUX05AF9BY9725RCJTN2XJGSQHBSKS9XHDF5NB79CB9TLRNNM6HMKY"
                                                                                      + "KNVR24KUK9GRAT96BM8Q5WAEP6DPVX9E9S2GGVV2DS00DMDAP3SGHA6FJR00DMDAP"
                                                                                      + "3SGHA6FJR"
                                                                                  },
                                                                                  {
                                                                                      "Enigma:EncryptionConstant",
                                                                                      "-1436100175"
                                                                                  }
                                                                              };

        /// <summary>
        /// The service.
        /// </summary>
        private readonly KeyService service;

        #endregion Private Fields

        #region Public Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="KeyServiceTests"/> class.
        /// </summary>
        public KeyServiceTests()
        {
            var builder = new ConfigurationBuilder();
            builder.AddInMemoryCollection(EnigmaConfig);

            this.service = new KeyService(builder.Build());
        }

        #endregion Public Constructors

        #region Public Methods

        /// <summary>
        /// Should generate a key with a valid ID.
        /// </summary>
        [Fact]
        public void ShouldGenerateKey()
        {
            // Assert
            Assert.Null(this.service.GenerateKey("Example User", "566BF-6A09D-01491-8D0E0-EF2D0-F0980-54583-3F614", DateTime.UtcNow.Date.AddDays(3)));
        }

        #endregion Public Methods
    }
}